/*
 * webfilter_dump.c -- dump the atabase to standard output
 *
 * (c) 2003 Dr. Andreas Mueller, Beratung und Entwicklung
 */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <gdbm.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

int	debug = 0;
int	nocomment = 0;

void	print_value(char *hostname, datum entry, datum comment) {
	char	*cat;
	int	off;

	printf("%s", hostname);
	cat = (char *)entry.dptr;
	for (off = 0; off < entry.dsize; off += strlen(cat + off) + 1) {
		printf(" %s", cat + off);
	}
	if ((!nocomment) && (comment.dptr)) {
		printf("#%s", (char *)comment.dptr);
	}
	printf("\n");
}

int	main(int argc, char *argv[]) {
	int		c, keysize;
	GDBM_FILE	db;
	char		*dbname;
	datum		key, commentkey, entry, comment;

	/* parse command line						*/
	while (EOF != (c = getopt(argc, argv, "dc")))
		switch (c) {
		case 'd':
			debug++;
			break;
		case 'c':
			nocomment = 1;
			break;
		}

	/* next argument is database					*/
	if ((argc - 1) > optind) {
		fprintf(stderr, "%s: must specify database name\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	dbname = argv[optind];
	if (debug)
		fprintf(stderr, "%s:%d: database: %s\n", __FILE__, __LINE__,
			dbname);

	/* open database						*/
	if (NULL == (db = gdbm_open(dbname, 0, O_RDONLY, 0666, NULL))) {
		fprintf(stderr, "%s: cannot open database %s: %s (%d)\n",
			argv[0], dbname, strerror(errno), errno);
		exit(EXIT_FAILURE);
	}

	/* iterate through the database					*/
	for (key = gdbm_firstkey(db); NULL != key.dptr; key = gdbm_nextkey(db, key)) {
		/* is this a comment key?				*/
		if (strchr((char *)key.dptr, '#')) {
			if (debug)
				fprintf(stderr, "%s:%d: skipping comment "
					"key %s\n", __FILE__, __LINE__,
					(char *)key.dptr);
			continue;
		}

		/* get the categories entry				*/
		keysize = strlen((char *)key.dptr) + 1;
		if (debug)
			fprintf(stderr, "%s:%d: entry for key %s found\n",
				__FILE__, __LINE__, (char *)key.dptr);
		entry = gdbm_fetch(db, key);

		/* try to also fetch a comment entry			*/
		commentkey.dsize = keysize + 1;
		commentkey.dptr = (char *)malloc(commentkey.dsize);
		strcpy((char *)commentkey.dptr, (char *)key.dptr);
		strcat((char *)commentkey.dptr, "#");
		comment = gdbm_fetch(db, commentkey);
		free(commentkey.dptr);

		/* display the line					*/
		print_value((char *)key.dptr, entry, comment);
	}

	/* close the database						*/
	if (debug)
		fprintf(stderr, "%s:%d: closing database %s\n", __FILE__,
			__LINE__, dbname);
	gdbm_close(db);
	exit(EXIT_FAILURE);
}
